﻿//
//   (c) Copyright Agilent Technologies 2012
//              All rights reserved.
//
//  Customer shall have the personal, non-
// transferable rights to use, copy or modify
// this SAMPLE PROGRAM for Customer's internal
// operations. Customer shall use the SAMPLE
// PROGRAM solely and exclusively for its own
// purpose and shall not license, lease, market
// or distribute the SAMPLE PROGRAM or modification
// or any part thereof.
//
//  Agilent shall not be liable for the quality,
// performance or behavior of the SAMPLE PROGRAM.
// Agilent especially disclaims that the operation
// of the SAMPLE PROGRAM shall be uninterrupted or
// error free. This SAMPLE PROGRAM is provided
// AS IS.
//
//  AGILENT DISCLAIMS THE IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE.
//
//  Agilent shall not be liable for any infringement
// of any patent, trademark, copyright or other
// proprietary rights by the SAMPLE PROGRAM or
// its use. Agilent does not warrant that the SAMPLE
// PROGRAM is free from infringements or such
// rights of third parties. However, Agilent will not
// knowingly infringe or deliver a software that
// infringes the patent, trademark, copyright or
// other proprietary right of a third party.
//
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Ivi.Visa.Interop;

namespace B2900CsSample
{
    class SmuControl
    {
        private ResourceManager ioMgr = new ResourceManager();
        private FormattedIO488 instrument = null;
        private string visaResStr = "";
        private int maxChannel = 0;
        private string modelName = "";
        private string errorMsg = "";
        private const int WAIT_BEFORE_QUERY = 10;

        //Public Sub New()
        //End Sub

        public const int COLUMN_COUNT = 5;

        public int MaxSmuChannel
        {
            get {return maxChannel;}
        }

        public string ModelName
        {
            get {return modelName;}
        }

        public string ErrorMessage
        {
            get {return errorMsg;}
        }


        /// <summary>
        /// Open connection with B2900A
        /// </summary>
        /// <param name="address"></param>
        /// <returns></returns>
        public bool OpenDevice(string address)
        {
            try
            {
                if (instrument != null) {
                    CloseDevice();
                }

                string buf;
                
                // Open
                visaResStr = address;
                instrument = new FormattedIO488();
                instrument.IO = (IMessage)ioMgr.Open(address);
                instrument.IO.TerminationCharacterEnabled = true;
                instrument.IO.Timeout = 100000;
                System.Threading.Thread.Sleep(200);

                // Check Model Name
                instrument.WriteString("*IDN?");
                System.Threading.Thread.Sleep(WAIT_BEFORE_QUERY);
                buf = instrument.ReadString().Trim();
                string[] tokens = buf.Split(',');
                if (tokens.Length < 2)
                {
                    maxChannel = 0;
                    modelName = "";
                    errorMsg = "Illegal *IDN response.";
                    CloseDevice();
                    return false;
                }

                modelName = buf.Split(',')[1];
                switch (modelName)
                {
                    case "B2901A":
                    case "B2911A":
                    case "B2961A":
                        maxChannel = 1;
                        break;
                    case "B2902A":
                    case "B2912A":
                    case "B2962A":
                        maxChannel = 2;
                        break;
                    default:
                        maxChannel = 0;
                        errorMsg = "Unknown model is found.";
                        CloseDevice();
                        return false;
                }
 
                // Check Language Mode
                instrument.WriteString(":SYST:LANG?");
                System.Threading.Thread.Sleep(WAIT_BEFORE_QUERY);
                buf = instrument.ReadString().Trim();
                if (buf != "\"DEF\"")
                {
                    maxChannel = 0;
                    errorMsg = "Not support 2400 language mode.";
                    CloseDevice();
                    return false;
                }

                // Reset
                instrument.WriteString("*RST");
                instrument.WriteString("*CLS");
            }
            catch ( Exception ex )
            {
                instrument = null;
                errorMsg = ex.Message;
                return false;
            }
            return true;
        }


        /// <summary>
        /// Close connection with B2900A
        /// </summary>
        public void CloseDevice()
        {
            if (instrument != null) instrument.IO.Close();
            instrument = null;
        }


        /// <summary>
        /// Check error log.
        /// </summary>
        /// <returns></returns>
        private bool ErrorCheck()
        {
            try
            {
                string ret;
                instrument.WriteString(":SYST:ERR:COUNT?");
                System.Threading.Thread.Sleep(WAIT_BEFORE_QUERY);
                ret = instrument.ReadString().Trim();
                if ( int.Parse(ret) > 0 )
                {
                    instrument.WriteString(":SYST:ERR:ALL?");
                    System.Threading.Thread.Sleep(WAIT_BEFORE_QUERY);
                    errorMsg = instrument.ReadString();
                    return false;
                }
            }
            catch ( Exception ex )
            {
                errorMsg = ex.Message;
                return false;
            }
            return true;
        }


        /// <summary>
        /// Setting for sweep channel.
        /// </summary>
        /// <param name="ch">B2960A Channel</param>
        /// <param name="startC">Start Current</param>
        /// <param name="stopC">Stop Current</param>
        /// <param name="NofStep">Num of steps</param>
        /// <param name="compV">Compliance of Voltage</param>
        /// <param name="sweepType">Sweep method [1-3]</param>
        /// <returns></returns>
        public bool SetSweepChannel(int ch, double startC, double stopC, int NofStep, double compV, double measPlc, int sweepType)
        {
            bool ret = false;

            try
            {
                instrument.WriteString(":SOUR" + ch.ToString() + ":FUNC:MODE CURR");                        // Set source : Current
                instrument.WriteString(":SOUR" + ch.ToString() + ":FUNC:SHAP DC");                          // Output shape : DC
                instrument.WriteString(":SENS" + ch.ToString() + ":FUNC:ON \"VOLT\",\"CURR\"");             // Enable measurement : Voltage, Current

////            The following two lines are for [B2912A].  / [B2960A] : Not supported    
//              instrument.WriteString(":SENS" + ch.ToString() + ":VOLT:RANG:AUTO OFF");                    // Disables the automatic ranging function.
//              instrument.WriteString(":SENS" + ch.ToString() + ":VOLT:RANG " + compV.ToString());         // Set voltage output range

                instrument.WriteString(":SENS" + ch.ToString() + ":CURR:APER:AUTO OFF");                    // Disables the automatic aperture function.
                instrument.WriteString(":SENS" + ch.ToString() + ":CURR:DC:NPLC " + measPlc.ToString());    // Set measure time

                instrument.WriteString(":SOUR" + ch.ToString() + ":CURR " + startC.ToString());             // Set output Current
                instrument.WriteString(":SENS" + ch.ToString() + ":VOLT:PROT " + compV.ToString());         // Compliance of Voltage


                // Sweep params.
                switch (sweepType)
                {
                    case 1:     // Normal
                        ret = SetSweepParamNormal(ch, startC, stopC, NofStep);
                        break;

                    case 2:     // NULL
                        ret = SetSweepParamNull(ch, startC, stopC, NofStep);
                        break;

                    case 3:     // Forward/Reverse
                        ret = SetSweepParamFwRv(ch, startC, stopC, NofStep);
                        break;

                    default:
                        break;
                }
                if (!ret) return ret;

                // ARM params.
                instrument.WriteString(":ARM" + ch.ToString() + ":COUN 1");
                instrument.WriteString(":ARM" + ch.ToString() + ":DEL 0.0");
                instrument.WriteString(":ARM" + ch.ToString() + ":SOUR AINT");
                // Trigger params.
                instrument.WriteString(":TRIG" + ch.ToString() + ":COUN " + (NofStep * sweepType).ToString());
                instrument.WriteString(":TRIG" + ch.ToString() + ":DEL 0.0");
                instrument.WriteString(":TRIG" + ch.ToString() + ":SOUR AINT");

                instrument.WriteString(":OUTP" + ch.ToString() + ":STAT ON");                           // Output ON


                // External Trigger
                instrument.WriteString(":SOUR" + ch.ToString() + ":TOUT:SIGN EXT9");                    // Set Trigger Out : External 9-pin
                instrument.WriteString(":SOUR" + ch.ToString() + ":DIG:EXT9:FUNC TOUT");                // TriggerOUT
                instrument.WriteString(":SOUR" + ch.ToString() + ":DIG:EXT9:TOUT:POS AFT");             // Output Trigger Timing : after
                instrument.WriteString(":SOUR" + ch.ToString() + ":DIG:EXT9:TOUT:WIDT 1E-4");           // Pulse width 0.0001[sec] (100us)
                instrument.WriteString(":SOUR" + ch.ToString() + ":TOUT 1");                            // Enables the trigger output.

                return ErrorCheck();
            }
            catch ( Exception ex )
            {
                errorMsg = ex.Message;
                return false;
            }
        }


        /// <summary>
        /// Set sweep parameters for Normal method
        /// </summary>
        /// <param name="ch">B2960A Channel</param>
        /// <param name="startC">Start Current</param>
        /// <param name="stopC">Stop Current</param>
        /// <param name="NofStep">Num of steps</param>
        /// <returns>true/false</returns>
        private bool SetSweepParamNormal(int ch, double startC, double stopC, int NofStep)
        {
            try
            {
                instrument.WriteString(":SOUR" + ch.ToString() + ":CURR:MODE SWE");                     // Current Output mode : Sweep

                instrument.WriteString(":SOUR" + ch.ToString() + ":SWE:DIR UP");                        // Direction : UP
                instrument.WriteString(":SOUR" + ch.ToString() + ":SWE:STA SING");                      // Stair     : Single
                instrument.WriteString(":SOUR" + ch.ToString() + ":SWE:RANG AUTO");                     // Ranging   : Auto
                instrument.WriteString(":SOUR" + ch.ToString() + ":SWE:SPAC LIN");                      // Spacing   : Linear

                instrument.WriteString(":SOUR" + ch.ToString() + ":CURR:STAR " + startC.ToString());    // Set Start Voltage
                instrument.WriteString(":SOUR" + ch.ToString() + ":CURR:STOP " + stopC.ToString());     // Set Stop Voltage

                instrument.WriteString(":SOUR" + ch.ToString() + ":SWE:POIN " + NofStep.ToString());

                return true;
            }
            catch (Exception ex)
            {
                errorMsg = ex.Message;
                return false;
            }
        }


        /// <summary>
        /// Set sweep parameters for NULL method
        /// </summary>
        /// <param name="ch">B2960A Channel</param>
        /// <param name="startC">Start Current</param>
        /// <param name="stopC">Stop Current</param>
        /// <param name="NofStep">Num of steps</param>
        /// <returns>true/false</returns>
        private bool SetSweepParamNull(int ch, double startC, double stopC, int NofStep)
        {
            string listData = "";
            double currnt;
            double cStep = (stopC - startC) / ((double)(NofStep - 1));

            // Make List
            currnt = startC;
            listData = "0," + currnt.ToString();
            for (int i = 1; i < NofStep; i++)
            {
                currnt += cStep;
                listData += ",0," + currnt.ToString();
            }

            try
            {
                instrument.WriteString(":SOUR" + ch.ToString() + ":CURR:MODE LIST");                // Current Output mode : List
                instrument.WriteString(":SOUR" + ch.ToString() + ":LIST:CURR " + listData);         // Set List data

                return true;
            }
            catch (Exception ex)
            {
                errorMsg = ex.Message;
                return false;
            }
        }


        /// <summary>
        /// Set sweep parameters for Forward/Reverse method
        /// </summary>
        /// <param name="ch">B2960A Channel</param>
        /// <param name="startC">Start Current</param>
        /// <param name="stopC">Stop Current</param>
        /// <param name="NofStep">Num of steps</param>
        /// <returns>true/false</returns>
        private bool SetSweepParamFwRv(int ch, double startC, double stopC, int NofStep)
        {
            string listData = "";
            double currnt;
            double cStep = (stopC - startC) / ((double)(NofStep - 1));

            // Make List
            currnt = startC;
            listData = "0," + currnt.ToString() + ",-" + currnt.ToString();
            for (int i = 1; i < NofStep; i++)
            {
                currnt += cStep;
                listData += ",0," + currnt.ToString() + ",-" + currnt.ToString();
            }

            try
            {
                instrument.WriteString(":SOUR" + ch.ToString() + ":CURR:MODE LIST");                // Current Output mode : List
                instrument.WriteString(":SOUR" + ch.ToString() + ":LIST:CURR " + listData);         // Set List data

                return true;
            }
            catch (Exception ex)
            {
                errorMsg = ex.Message;
                return false;
            }
        }


        /// <summary>
        /// Initiates B2900A
        /// </summary>
        /// <param name="ch"></param>
        /// <returns></returns>
        public bool Initiate( int[] ch )
        {
            try
            {
                instrument.WriteString(":FORM ASC");                                            // Set Format : ASCII
                instrument.WriteString(":FORM:ELEM:SENS VOLT,CURR,TIME,STAT,SOUR");             // Data Format
                string chlist = "(@";
                for ( int i = 0 ; i < ch.Length ; i++ ) 
                {
                    if ( i > 0 ) chlist += ",";
                    chlist += ch[i].ToString();
                }
                chlist += ")";
                instrument.WriteString(":INIT " + chlist);
                return ErrorCheck();
            }
            catch ( Exception ex )
            {
                errorMsg = ex.Message;
                return false;
            }
        }


        /// <summary>
        /// Get measure results
        /// </summary>
        /// <param name="ch"></param>
        /// <param name="fromStart"></param>
        /// <param name="len"></param>
        /// <param name="data"></param>
        /// <param name="actrow"></param>
        /// <returns></returns>
        public bool GetData( int ch, bool fromStart, int len, ref double[,] data, out int actrow )
        {
            actrow = 0;
            try
            {
                string ret;
                bool isBusy;
                do
                {
                    instrument.WriteString(":STAT:OPER:COND?");
                    System.Threading.Thread.Sleep(WAIT_BEFORE_QUERY);
                    ret = instrument.ReadString().Trim();
                    isBusy = ((int.Parse(ret) & 0x0010) == 0);

                    instrument.WriteString(":SENS" + ch.ToString() + ":DATA? " + (fromStart ? "STAR," : "CURR,") + len.ToString());
                    System.Threading.Thread.Sleep(WAIT_BEFORE_QUERY);
                    ret = instrument.ReadString().Trim();
                    string[] ary = ret.Split(',');

                    int row, col;
                    for (row = 0; row < ary.Length / COLUMN_COUNT; row++)
                    {
                        if (double.Parse(ary[row * COLUMN_COUNT]) >= 9.91e37) break;
                        for (col = 0; col < COLUMN_COUNT; col++)
                        {
                            data[row,col] = double.Parse(ary[row * COLUMN_COUNT + col]);
                        }
                    }
                    actrow = row;

                } while (actrow == 0 && isBusy);

                return ErrorCheck();
            }
            catch ( Exception ex )
            {
                errorMsg = ex.Message;
                return false;
            }
        }
    }
}
